#############################################################################
##  
#W  factint/FactorsIntUtil.g           FUtil                    Frank Lbeck 
##  
#Y  Copyright (C)  2004
##  
##  This file add some functionality to the 'FactInt' package by Stefan Kohl:
##  It is only used when 'FactInt' could be successfully loaded.
##  
##  #* this is now directly in FactInt *#
##  (a) It uses data files which provide prime factors of numbers of form 
##          a^n - 1.
##      These are collected and computed by Richard Brent and many others.
##      Factorizations of such numbers (in particular when 'a' is a prime)
##      are important in GAP for computing orders of matrices over finite 
##      fields or elements in huge finite fields.
##  
##      See http://web.comlab.ox.ac.uk/oucl/work/richard.brent/factors.html
##      and the links on that page.
##      
##  
##  (b) There is an interface to the program 'ppsiqs' by Satoshi Tomabechi
##      (see http://www.asahi-net.or.jp/~KC2H-MSM/cn/index.htm, available for
##      PCs with Linux or Windows).
##  
##      It is very good on large numbers with large factors and could be used 
##      by 'FactInt' instead of the functions 'Factors{ECM,MPQS,CFRAC}' (after
##      removing 'small' factors by other methods).
##  
##      If the input is a prime 'ppsiqs' *proves* the primality by APR-CL.
##      
FUtil.StoreRevision( "factint/FactorsIntUtil.g",
   "$Id: FactorsIntUtil.g 37 2006-08-16 10:45:53Z luebeck $" );

if LoadPackage("FactInt") = true then


##  'FactorsPPSIQS( <n> )' takes an integer <n> and tries to find a factor
##  of <n> using the external program 'ppsiqs' which must be in the users
##  executables path. 
##  
##  It returns 'fail' in case of any problem. 
##  It returns <n> itself when it was *proven* (by the APR-CL 
##  algorithm that <n> is prime. 
##  Otherwise a list [a, n/a] of two proper factors if <n> is returned.
##  
#* we would need an ECM algorithm as well and then could substitute the ECM
#  and MPQS calls in FactInt by these external programs.  *#
BindGlobal("FactorsPPSIQS", function(n)
  local cmd, res, inp, out, pos, fac;
  cmd := Filename(DirectoriesSystemPrograms(), "ppsiqs");
  if not IsString(cmd) then
    return fail;
  fi;
  res := "";
  inp := InputTextString(String(n));
  out := OutputTextString(res, false);
  Process(DirectoryCurrent(), cmd, inp, out, ["256"]);
  CloseStream(inp);
  CloseStream(out);
  # check if a proper factor was found
  pos := PositionSublist(res, "\nfound  factor ");
  if pos <> fail then
    fac := Int(res{[pos+15..Position(res, '\n', pos+14)-1]});
    return [fac, n/fac];
  fi;
  # check if n was proven to be prime
  pos := PositionSublist(res, "\nfinal test ");  
  if pos <> fail then
    return n;
  fi;
  return fail;
end);



fi;



BindGlobal("IsPrimeByECPP", function(n)
  local cmd, res, inp, out, pos, fac;
  cmd := Filename(DirectoriesSystemPrograms(), "xrunecpp");
  if not IsString(cmd) then
    return fail;
  fi;
  res := "";
  inp := InputTextString(String(n));
  out := OutputTextString(res, false);
  Process(DirectoryCurrent(), cmd, inp, out, ["-Q", "-f", "-"]);
  CloseStream(inp);
  CloseStream(out);
  pos := PositionSublist(res, "1Enter");
  if pos <> fail then
    return true;
  fi;
  pos := PositionSublist(res, "0Enter");
  if pos <> fail then
    return false;
  fi;
  return fail;
end);

