
# here are a few examples of using the functionality of Decimal.g*

# get a decimal approximation (default is 10 digit after the decimal point)
a1 := DecimalApproximation(3/8);
# also works for non-real numbers, optional 2nd argument gives the precision
a2 := DecimalApproximation(E(3), 100);
# splitting in real and imaginary part
a3 := RealPart(a2);
a4 := ImaginaryPart(a2);
# the basic data in a4 are its Mantissa, Precision and Epsilon, these mean that
# the number represents some real number in the range 
#     (Mantissa(a4) +/- Epsilon(a4)) * 10^-Precision(a4)
Mantissa(a4);
Precision(a4);
Epsilon(a4);
# basic arithmetic for approximate numbers is implemented,
# in this example the precision of the result is the precision of the less
# precise operand
a5 := a4 - a1;
# comparison of decimal approximations:
a6 := DecimalApproximation(1/2);
a7 := RealPart(DecimalApproximation(Sqrt(3)));
# this gives true (or false) if a6 < a7 (or a7 < a6) in the sense that the
# intervals of precision don't overlap 
TruelyLessDecimal(a6,a7);
# ... and it gives fail otherwise:
a:=RealPart(DecimalApproximation(Sqrt(3)));
for i in [1..20] do a := a*a/a; od;
TruelyLessDecimal(a6,a);
# the following funtion returns true (or false) if the real part of a GAP 
# cyclotomic number (a rational linear combination of E(n)'s) is positive
# (or negative), this is achived by increasing the precision until the
# question can be decided
HasPositiveRealPartCyc(Sqrt(3)-1/2);
# there are algorithms for the decimal approximation of square roots and \pi
SqrtDecimalApproximation(3,1000);
PiDecimalApproximation(1000);



# here is an example for HasPositiveRealPartCyc with a number very close to
# zero:

# produce a few positive real numbers
a:=E(5)+E(5)^-1;
b:=E(7)+E(7)^-1;
c:=E(11)+E(11)^-1;
d:=E(12)+E(12)^-1;

# find complex decimal approximations with 100 decimals
ad:=DecimalApproximation(a,100);
bd:=DecimalApproximation(b,100);
cd:=DecimalApproximation(c,100);
dd:=DecimalApproximation(d,100);

# look at first 50 decimals
a1:=Int(Mantissa(RealPart(ad))/10^50);
b1:=Int(Mantissa(RealPart(bd))/10^50);
c1:=Int(Mantissa(RealPart(cd))/10^50);
d1:=Int(Mantissa(RealPart(dd))/10^50);

# find a linear combination very close to zero
RequirePackage("EDIM");
g:=GcdexIntLLL(a1,b1,c1,d1);

# compute this linear combination for the original numbers (ugly expression 
# in E(4620)) ...
z:=g[2]*[a,b,c,d];
# ... and the same with the approximations
zd:=g[2]*[ad,bd,cd,dd];
Precision(RealPart(zd));
Epsilon(RealPart(zd));
# the last two commands show that z is negative

# The following gets the same result from the complicated expression for z,
# it automatically increases the precision to find the answer.
HasPositiveRealPartCyc(z);



